<?php

namespace EventPortal\Domain\Agenda\Requests;

use Carbon\Carbon;
use Illuminate\Foundation\Http\FormRequest;
use Illuminate\Validation\Rule;

class StoreAgendaRequest extends FormRequest
{
    public function authorize(): bool
    {
        return true;
    }

    protected function prepareForValidation(): void
    {
        $this->merge([
            'dates' => $this->formattedDates(),
        ]);
    }

    public function rules(): array
    {
        return [
            'dates.*' => [
                Rule::unique('agendas', 'date')->where('event_id', $this->event->id),
            ],
        ];
    }

    public function messages(): array
    {
        return [
            'dates.*.unique' => 'The date must be unique',
        ];
    }

    private function formattedDates(): array
    {
        return array_map(function (string $date) {
            return Carbon::createFromFormat('d/m/Y', $date)->format('Y-m-d');
        }, $this->dates);
    }
}
