<?php

namespace EventPortal\Domain\Event;

use EventPortal\Contracts\Event;
use EventPortal\Domain\Component\Button;
use EventPortal\Domain\Event\Form\Models\Form;
use EventPortal\Domain\Event\Models\EventDelegate;
use EventPortal\Domain\EventDelegate\RegistrationStatus;
use EventPortal\Enums\InertiaMethod;

class DeclineCallToAction
{
    private Event $event;

    private ?Form $form;

    private string $status;

    public static function make(): self
    {
        return new self();
    }

    public function forEventDelegate(EventDelegate $eventDelegate): self
    {
        $this->event = $eventDelegate->event;
        $this->form = Form::forEvent($this->event)->decline()->first();

        $this->status = $eventDelegate->status;

        return $this;
    }

    public function shouldShow(): bool
    {
        if ($this->form === null) {
            return false;
        }

        return in_array($this->status, [
            RegistrationStatus::invited(),
            RegistrationStatus::registered(),
            RegistrationStatus::registering(),
            RegistrationStatus::declining(),
        ]);
    }

    public function get(): ?Button
    {
        if ($this->shouldShow() === false) {
            return null;
        }

        return Button::make('Decline invite')
            ->variant('secondary')
            ->method(InertiaMethod::VISIT())
            ->route('delegate.form.index', [
                'event' => $this->event->slug,
                'form' => $this->form->slug,
            ]);
    }
}
