<?php

namespace EventPortal\Tests\Unit\ActionList;

use EventPortal\Domain\ActionList\ActionList;
use EventPortal\Domain\ActionList\Task;
use EventPortal\Domain\Event\Models\Event;
use EventPortal\Domain\User\Models\User;
use EventPortal\Tests\TestCase;
use Illuminate\Support\Collection;

class ActionListTest extends TestCase
{
    /** @test **/
    public function can_generate()
    {
        $actionList = ActionList::state('draft')->get();

        $this->assertSame('draft', $actionList->status);
    }

    /** @test **/
    public function can_add_type()
    {
        $actionList = ActionList::state('draft')->type('a-type')->get();

        $this->assertSame('draft', $actionList->status);
        $this->assertSame('a-type', $actionList->type);
    }

    /** @test **/
    public function can_add_task()
    {
        $actionList = ActionList::state('draft')
            ->addTask(function () {
                return [
                    Task::make()->label('Task name'),
                ];
            })
            ->get();

        $this->assertInstanceOf(Collection::class, $actionList->tasks);

        $task = $actionList->tasks->first();

        $this->assertSame('task_name', $task->name);
        $this->assertSame('Task name', $task->label);
    }

    /** @test **/
    public function can_add_multiple_tasks()
    {
        $actionList = ActionList::state('draft')
            ->addTask(function () {
                return [
                    Task::make()->label('Task name'),
                ];
            })
            ->addTask(function () {
                return [
                    Task::make()->label('Another Task'),
                ];
            })
            ->get();

        $this->assertInstanceOf(Collection::class, $actionList->tasks);

        $task = $actionList->tasks->first();
        $this->assertSame('task_name', $task->name);
        $this->assertSame('Task name', $task->label);

        $task = $actionList->tasks->last();
        $this->assertSame('another_task', $task->name);
        $this->assertSame('Another Task', $task->label);
    }

    /** @test **/
    public function can_add_context()
    {
        $event = factory(Event::class)->make([
            'name' => 'Event Name',
        ]);

        $actionList = ActionList::state('draft')
            ->context($event)
            ->addTask(function (Event $event) {
                return [
                    Task::make()->label($event->name),
                ];
            })
            ->get();

        $this->assertSame('Event Name', $actionList->tasks->first()->label);
    }

    /** @test **/
    public function can_add_multiple_to_context()
    {
        $event = factory(Event::class)->make([
            'name' => 'Event Name',
        ]);

        $user = factory(User::class)->make([
            'email' => 'jeff@example.com',
        ]);

        $actionList = ActionList::state('draft')
            ->context($event, $user)
            ->addTask(function (Event $event, User $user) {
                return [
                    Task::make()->label($event->name . ' ' . $user->email),
                ];
            })
            ->get();

        $this->assertSame(
            'Event Name jeff@example.com',
            $actionList->tasks->first()->label
        );
    }
}
