<?php

namespace EventPortal\Tests\Unit\Util;

use EventPortal\Domain\Component\Button;
use EventPortal\Domain\Event\Models\Event;
use EventPortal\Domain\Util\QuickButtons\Facades\QuickButtons;
use EventPortal\Tests\TestCase;
use Illuminate\Foundation\Testing\RefreshDatabase;
use Spatie\Snapshots\MatchesSnapshots;

class QuickButtonTest extends TestCase
{
    use RefreshDatabase, MatchesSnapshots;

    /** @test **/
    public function can_generate_quick_buttons()
    {
        $buttons = QuickButtons::make('events.test')
            ->addButtons(function (): array {
                return [
                    Button::make('Hello World!'),
                ];
            })
            ->get();

        $this->assertMatchesJsonSnapshot($buttons);
    }

    /** @test **/
    public function can_add_context()
    {
        $event = factory(Event::class)->state('snapshot')->create();

        QuickButtons::make('events.test')
            ->addButtons(function (Event $event): array {
                return [
                    Button::make("{$event->name}"),
                ];
            });

        $buttons = QuickButtons::make('events.test')->context($event)->get();

        $this->assertMatchesJsonSnapshot($buttons);
    }
}
